package com.adc.da.util.utils;

import com.adc.da.util.exception.AdcDaBaseException;
import com.alibaba.druid.pool.DruidDataSource;
import org.springframework.core.env.Environment;

import javax.sql.DataSource;

public class DataSourceUtils {

    public static DataSource createDruidDataSource(Environment env) {
        try {
            DruidDataSource dataSource = new DruidDataSource();
            dataSource.setDriverClassName(env.getProperty("spring.datasource.driver-class-name"));
            dataSource.setUrl(env.getProperty("spring.datasource.url"));
            dataSource.setUsername(env.getProperty("spring.datasource.username"));
            dataSource.setPassword(env.getProperty("spring.datasource.password"));
            dataSource.setInitialSize(Integer.parseInt(env.getProperty("spring.datasource.initialSize")));
            dataSource.setMaxActive(Integer.parseInt(env.getProperty("spring.datasource.maxActive")));
            dataSource.setMinIdle(Integer.parseInt(env.getProperty("spring.datasource.minIdle")));
            dataSource.setMaxWait(Integer.parseInt(env.getProperty("spring.datasource.maxWait")));
            dataSource.setTimeBetweenEvictionRunsMillis(Integer.parseInt(env.getProperty("spring.datasource.timeBetweenEvictionRunsMillis")));
            dataSource.setMinEvictableIdleTimeMillis(Integer.parseInt(env.getProperty("spring.datasource.minEvictableIdleTimeMillis")));
            dataSource.setValidationQuery(env.getProperty("spring.datasource.validationQuery"));
            String testOnBorrow = env.getProperty("spring.datasource.testOnBorrow");
            if (testOnBorrow != null) {
                dataSource.setTestOnBorrow(Boolean.valueOf(testOnBorrow));
            }
            String testWhileIdle = env.getProperty("spring.datasource.testWhileIdle");
            if (testWhileIdle != null) {
                dataSource.setTestWhileIdle(Boolean.valueOf(testWhileIdle));
            }
            String poolPreparedStatements = env.getProperty("spring.datasource.poolPreparedStatements");
            if (poolPreparedStatements != null) {
                dataSource.setPoolPreparedStatements(Boolean.valueOf(poolPreparedStatements));
            }
            String testOnReturn = env.getProperty("spring.datasource.testOnReturn");
            if (testOnReturn != null) {
                dataSource.setTestOnReturn(Boolean.valueOf(testOnReturn));
            }

            String maxPoolPreparedStatementPerConnectionSize = env.getProperty("spring.datasource.maxPoolPreparedStatementPerConnectionSize");
            if (maxPoolPreparedStatementPerConnectionSize != null) {
                dataSource.setMaxPoolPreparedStatementPerConnectionSize(Integer.valueOf(maxPoolPreparedStatementPerConnectionSize));
            }
            dataSource.setFilters(env.getProperty("spring.datasource.filters"));
            dataSource.setConnectionProperties(env.getProperty("spring.datasource.connectionProperties"));
            String useGlobalDataSourceStat = env.getProperty("spring.datasource.useGlobalDataSourceStat");
            if (useGlobalDataSourceStat != null) {
                dataSource.setUseGlobalDataSourceStat(Boolean.valueOf(useGlobalDataSourceStat));
            }

            return dataSource;
        } catch (Exception e) {
            e.printStackTrace();

            return null;
        }
    }

}
