package com.adc.da.util.utils;


import cn.afterturn.easypoi.excel.ExcelExportUtil;
import cn.afterturn.easypoi.excel.ExcelImportUtil;
import cn.afterturn.easypoi.excel.entity.ExportParams;
import cn.afterturn.easypoi.excel.entity.ImportParams;
import cn.afterturn.easypoi.excel.entity.enmus.ExcelType;
import org.apache.commons.lang3.StringUtils;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.util.List;
import java.util.NoSuchElementException;

public class ExcelUtils {

    /**
     * 用户信息导出类
     * @param response 响应
     * @param fileName 文件名
     * @param columnList 每列的标题名
     * @param dataList 导出的数据
     */
    public static void uploadExcelAboutUser(HttpServletResponse response,String fileName,List<String> columnList,List<List<String>> dataList){
        //1、创建一个工作簿
        Workbook workbook = new XSSFWorkbook();
        //2、创建一个工作表
        Sheet sheet = workbook.createSheet("参数信息");
        int excelRow = 0;
        //3、创建标题行 (1,1)
        Row titleRow = sheet.createRow(excelRow++); //0代表第一行
        for(int i = 1;i<=columnList.size();i++){
            //创建该行下的每一列，并写入标题数据
            Cell cell = titleRow.createCell(i);
            cell.setCellValue(columnList.get(i-1));
        }
        //设置内容行
        if(dataList!=null && dataList.size()>0){
            //序号是从1开始的
            int count = 1;
            //外层for循环创建行
            for(int i = 0;i<dataList.size();i++){
                Row dataRow = sheet.createRow(excelRow++);
                //内层for循环创建每行对应的列，并赋值
                for(int j = -1;j<dataList.get(0).size();j++){//由于多了一列序号列所以内层循环从-1开始
                    Cell cell = dataRow.createCell(j+1);
                    if(j==-1){//第一列是序号列，不是在数据库中读取的数据，因此手动递增赋值
                        cell.setCellValue(count++);
                    }else{//其余列是数据列，将数据库中读取到的数据依次赋值
                        cell.setCellValue(dataList.get(i).get(j));
                    }
                }
            }
        }
        long time = System.currentTimeMillis();

        OutputStream os = null;
        response.reset();
        //设置响应头
        setResponseHeader(response,fileName);
        try {
            //输出
            os = response.getOutputStream();
            workbook.write(os);
            System.out.println("导出参数信息.xlsx 完毕！");
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                // 关闭输出流
                if (os != null) {
                    os.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }


       /* //声明输出流
        OutputStream os = null;
        //设置响应头
        setResponseHeader(response,fileName);
        try {
            //获取输出流
            os = response.getOutputStream();
            //内存中保留1000条数据，以免内存溢出，其余写入硬盘
            SXSSFWorkbook wb = new SXSSFWorkbook(1000);
            //获取该工作区的第一个sheet
            Sheet sheet1 = wb.createSheet("sheet1");
            int excelRow = 0;
            //创建标题行
            Row titleRow = sheet1.createRow(excelRow++);
            for(int i = 0;i<columnList.size();i++){
                //创建该行下的每一列，并写入标题数据
                Cell cell = titleRow.createCell(i);
                cell.setCellValue(columnList.get(i));
            }
            //设置内容行
            if(dataList!=null && dataList.size()>0){
                //序号是从1开始的
                int count = 1;
                //外层for循环创建行
                for(int i = 0;i<dataList.size();i++){
                    Row dataRow = sheet1.createRow(excelRow++);
                    //内层for循环创建每行对应的列，并赋值
                    for(int j = -1;j<dataList.get(0).size();j++){//由于多了一列序号列所以内层循环从-1开始
                        Cell cell = dataRow.createCell(j+1);
                        if(j==-1){//第一列是序号列，不是在数据库中读取的数据，因此手动递增赋值
                            cell.setCellValue(count++);
                        }else{//其余列是数据列，将数据库中读取到的数据依次赋值
                            cell.setCellValue(dataList.get(i).get(j));
                        }
                    }
                }
            }
            //将整理好的excel数据写入流中
            wb.write(os);
        } catch (IOException e) {
            e.printStackTrace();
            e.getMessage();
        } finally {
            try {
                // 关闭输出流
                if (os != null) {
                    os.close();
                }
            } catch (IOException e) {
                e.printStackTrace();
            }
        }*/
    }

    /*
        设置浏览器下载响应头
     */
    private static void setResponseHeader(HttpServletResponse response, String fileName) {
        try {
            try {
                fileName = new String(fileName.getBytes("UTF8"),"ISO8859-1");
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            }
            response.setContentType("application/octet-stream;charset=UTF-8");
            response.setHeader("Content-Disposition", "attachment;filename="+ fileName);
            response.addHeader("Pargam", "no-cache");
            response.addHeader("Cache-Control", "no-cache");
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * 用户信息导出类
     * @param fileName 文件名
     * @param columnList 每列的标题名
     * @param dataList 导出的数据
     */
    public static void uploadExcelAboutUser(String fileName,List<String> columnList,List<List<String>> dataList){
        //1、创建一个工作簿
        Workbook workbook = new XSSFWorkbook();
        //2、创建一个工作表
        Sheet sheet = workbook.createSheet("参数信息");
        int excelRow = 0;
        //3、创建标题行 (1,1)
        Row titleRow = sheet.createRow(excelRow++); //0代表第一行
        for(int i = 1;i<=columnList.size();i++){
            //创建该行下的每一列，并写入标题数据
            Cell cell = titleRow.createCell(i);
            cell.setCellValue(columnList.get(i-1));
        }
        //设置内容行
        if(dataList!=null && dataList.size()>0){
            //序号是从1开始的
            int count = 1;
            //外层for循环创建行
            for(int i = 0;i<dataList.size();i++){
                Row dataRow = sheet.createRow(excelRow++);
                //内层for循环创建每行对应的列，并赋值
                for(int j = -1;j<dataList.get(0).size();j++){//由于多了一列序号列所以内层循环从-1开始
                    Cell cell = dataRow.createCell(j+1);
                    if(j==-1){//第一列是序号列，不是在数据库中读取的数据，因此手动递增赋值
                        cell.setCellValue(count++);
                    }else{//其余列是数据列，将数据库中读取到的数据依次赋值
                        cell.setCellValue(dataList.get(i).get(j));
                    }
                }
            }
        }

        try {
            //在本地生成一个excel文件
            File file = new File(fileName);
            FileOutputStream file1 = new FileOutputStream(file);
            workbook.write(file1);
            file1.close();
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace ();
        }
    }

    /**
     * Excel 类型枚举
     */
    enum ExcelTypeEnum {
        XLS("xls"), XLSX("xlsx");

        private String value;

        ExcelTypeEnum(String value) {
            this.value = value;
        }

        public String getValue() {
            return value;
        }

        public void setValue(String value) {
            this.value = value;
        }
    }

    /**
     * 功能描述：复杂导出Excel，包括文件名以及表名。创建表头
     *
     * @param list           导出的实体类
     * @param title          表头名称
     * @param sheetName      sheet表名
     * @param pojoClass      映射的实体类
     * @param isCreateHeader 是否创建表头
     * @return
     */
    public static Workbook exportExcel(List<?> list, String title, String sheetName, Class<?> pojoClass,
                                       boolean isCreateHeader) {
        ExportParams exportParams = new ExportParams(title, sheetName);
        exportParams.setType(ExcelType.XSSF);
        exportParams.setCreateHeadRows(isCreateHeader);
        return ExcelExportUtil.exportExcel(exportParams, pojoClass, list);
    }

    /**
     * 功能描述：复杂导出Excel，包括文件名以及表名。创建表头
     *
     * @param list           导出的实体类
     * @param exportParams   自定义表头
     * @param pojoClass      映射的实体类
     * @return
     */
    public static Workbook exportExcel(List<?> list, ExportParams exportParams, Class<?> pojoClass) {
        return ExcelExportUtil.exportExcel(exportParams, pojoClass, list);
    }

    /**
     * 功能描述：根据文件路径来导入Excel
     *
     * @param filePath   文件路径
     * @param titleRows  表标题的行数
     * @param headerRows 表头行数
     * @param pojoClass  Excel实体类
     * @return
     */
    public static <T> List<T> importExcel(String filePath, Integer titleRows, Integer headerRows, Class<T> pojoClass) {
        // 判断文件是否存在
        if (StringUtils.isBlank(filePath)) {
            return null;
        }
        ImportParams params = new ImportParams();
        params.setTitleRows(titleRows);
        params.setHeadRows(headerRows);
        List<T> list = null;
        try {
            list = ExcelImportUtil.importExcel(new File(filePath), pojoClass, params);
        } catch (NoSuchElementException e) {
            throw new RuntimeException("模板不能为空");
        } catch (Exception e) {
            e.printStackTrace();

        }
        return list;
    }

    /**
     * 功能描述：根据接收的Excel文件来导入Excel,并封装成实体类
     *
     * @param file       上传的文件
     * @param titleRows  表标题的行数
     * @param headerRows 表头行数
     * @param pojoClass  Excel实体类
     * @return
     */
    public static <T> List<T> importExcel(MultipartFile file, Integer titleRows, Integer headerRows,
                                          Class<T> pojoClass) {
        if (file == null) {
            return null;
        }
        ImportParams params = new ImportParams();
        params.setTitleRows(titleRows);
        params.setHeadRows(headerRows);
        List<T> list = null;
        try {
            list = ExcelImportUtil.importExcel(file.getInputStream(), pojoClass, params);
        } catch (NoSuchElementException e) {
            throw new RuntimeException("excel文件不能为空");
        } catch (Exception e) {
            throw new RuntimeException(e.getMessage());

        }
        return list;
    }

    public static void downLoadExcel(String fileName, HttpServletResponse response, Workbook workbook) {
        try {
            response.setCharacterEncoding("UTF-8");
            response.setHeader("content-Type", "application/vnd.ms-excel");
            response.setHeader("Content-Disposition",
                    "attachment;filename=" + URLEncoder.encode(fileName, "UTF-8"));
            workbook.write(response.getOutputStream());
        } catch (IOException e) {
            throw new RuntimeException(e.getMessage());
        }
    }
}
